﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DateDiff.DateDiff
{
    /// <summary>
    /// Utility class for dates consisting of year, month and day.
    /// </summary>
    static class DateDiffUtil
    {
        public static bool IsLeapYear(int year)
        {
            return (year % 4 == 0) && ( (year % 100 != 0) || (year % 400 == 0) );
        }

        public static bool IsEqual(int y1, int m1, int d1, int y2, int m2, int d2)
        {
            return ( y1 == y2 && m1 == m2 && d1 == d2);
        }

        public static bool IsBeforeThan(int y1, int m1, int d1, int y2, int m2, int d2)
        {
            return (y1 < y2 ? true : (y1 == y2 ? (m1 < m2 ? true : (m1 == m2 ? d1 < d2 : false)) : false));
        }

        public static bool IsAfterThan(int y1, int m1, int d1, int y2, int m2, int d2)
        {
            return (y1 > y2 ? true : (y1 == y2 ? (m1 > m2 ? true : (m1 == m2 ? d1 > d2 : false)) : false));
        }

        public static bool IsLastDayInMonth_Old(int year, int month, int day)
        {
            return ((month == 1 && day == 31) ||
             (IsLeapYear(year) ? (month == 2 && day == 29) : (month == 2 && day == 28)) ||
             (month == 3 && day == 31) ||
             (month == 4 && day == 30) ||
             (month == 5 && day == 31) ||
             (month == 6 && day == 30) ||
             (month == 7 && day == 31) ||
             (month == 8 && day == 31) ||
             (month == 9 && day == 30) ||
             (month == 10 && day == 31) ||
             (month == 11 && day == 30) ||
             (month == 12 && day == 31)
           );
        }

        public static bool IsLastDayInMonth(int year, int month, int day)
        {
            if (month < 1 || month > 12)
            {
                throw new ArgumentOutOfRangeException();
            }

            int[] daysPerMonth = { 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
            int[] daysPerMonthLeapYear = { 31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31 };
            if (IsLeapYear(year))
            {
                return (daysPerMonthLeapYear[month-1] == day);
            }
            else
            {
                return (daysPerMonth[month-1] == day);
            }
        }

        public static bool IsLastMonthInYear(int month)
        {
            return (month == 12);
        }

        public static bool IsMonthValid(int month)
        {
            return (month >= 1 && month <= 12);
        }
                
        public static bool IsDateValid(int year, int month, int day)
        {
            return  (day >= 1 &&
                    (month == 1 && day <= 31) || 
                    (IsLeapYear(year) ? (month == 2 && day <= 29) : (month == 2 && day <= 28) ) ||
                    (month == 3 && day <= 31) || 
                    (month == 4 && day <= 30) || 
                    (month == 5 && day <= 31) || 
                    (month == 6 && day <= 30) || 
                    (month == 7 && day <= 31) || 
                    (month == 8 && day <= 31) || 
                    (month == 9 && day <= 30) || 
                    (month == 10 && day <= 31) || 
                    (month == 11 && day <= 30) || 
                    (month == 12 && day <= 31)
                   ) ;
        }
    }
}
