﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DateDiff.DateDiff
{
    /// <summary>
    /// Abstract base class for an date difference calculation algorithm.
    /// </summary>
    abstract class IDateDiff
    {
        protected int yearPreceding;
        protected int monthPreceding;
        protected int dayPreceding;

        protected int yearSubsequent;
        protected int monthSubsequent;
        protected int daySubsequent;

        /// <summary>
        /// Constructor. Checks for validity of dates. Order of dates is not relevant.
        /// </summary>
        /// <param name="y1">year of date 1</param>
        /// <param name="m1">month of date 1</param>
        /// <param name="d1">day of date 1</param>
        /// <param name="y2">year of date 2</param>
        /// <param name="m2">month of date 2</param>
        /// <param name="d2">day of date 2</param>
        protected IDateDiff(int y1, int m1, int d1, int y2, int m2, int d2)
        {
            if (!DateDiffUtil.IsDateValid(y1, m1, d1))
            {
                throw new ArgumentException("Invalid date: " + y1 + "-" + m1 + "-" + d1);
            }

            if (!DateDiffUtil.IsDateValid(y2, m2, d2))
            {
                throw new ArgumentException("Invalid date: " + y2 + "-" + m2 + "-" + d2);
            }

            IsDate1AfterDate2 = DateDiffUtil.IsAfterThan(y1, m1, d1, y2, m2, d2);

            yearSubsequent = IsDate1AfterDate2 ? y1 : y2;
            monthSubsequent = IsDate1AfterDate2 ? m1 : m2;
            daySubsequent = IsDate1AfterDate2 ? d1 : d2;

            yearPreceding = IsDate1AfterDate2 ? y2 : y1;
            monthPreceding = IsDate1AfterDate2 ? m2 : m1;
            dayPreceding = IsDate1AfterDate2 ? d2 : d1;
        }

        /// <summary>
        /// Get actual difference of both dates in days.
        /// </summary>
        /// <returns></returns>
        public abstract int GetDifference();

        /// <summary>
        /// true if date 1 is later / after than date 2; false if dates are equal or date 1 lies before date 2.
        /// </summary>
        public bool IsDate1AfterDate2 { private set; get; }
    }
}
